function DOF = doftn(rls,mfs,x,domain,tnorm,par)
% DOFTN calculates degrees of fulfillment (DOF's) of the premises,
% using a user-supplied t-norm operator as the fuzzy logic AND.
%
% DOF = DOFTN(RLS,MFS,X,DOM,OP,PAR)
% Inputs:       RLS - rule matrix
%               MFS - membership function matrix
%               X   - input vector or matrix
%               DOM - (optional) domain for membership function
%                     defined point-wise
%               OP  - user-supplied t-norm operator (function must have
%                     the same synopsis as MATLAB function MIN,
%                     additional parameter can be passed for parametric
%                     t-norms.
%               PAR - t-norm parameter
%
% Output:       DOF - matrix of DOF's
%
% See also TXT2RLS, RLS2TXT, MFSET, MFGET, INFER, DOFMIN, DOFPROD.

% Copyright (c) Robert Babuska, 1993-94

if nargin < 4, domain = []; end;
if nargin < 5, tnorm = 'min'; end;
if nargin < 6, par = []; end;
[k,ninps] = size(x);
[nrules,dummy] = size(rls);
[nmfs,npoints] = size(mfs);
if length(domain) == 0; mfany = [0 0 0 0 0];            % continuous domain
else mfany = ones(1,npoints); end;                      % discrete domain
mfs = [mfs; mfany]; nmfs = nmfs + 1;
DOF = zeros(nrules,k);                                  % transposed DOF
w = rls(:,1); rls = rls(:,2:ninps+1)';                  % extract weights
aux = ones(ninps,k*nrules);                             % for rls expansion
grades = aux';                                          % prepare for grades
aux(:) = rls(:)*ones(1,k);                              % expand rls
rls = zeros(nrules,k*ninps);                            % ....
rls(:) = aux';                                          % ....
ANYmask = rls == 0;                                     % find zeros (ANY)
rls(ANYmask) = nmfs*grades(ANYmask);                    % index for ANY
mask = rls < 0;                                         % find NOT in rules
sgn = -mask + (rls >= 0);                               % not(A) = 1 - A
shift = ones(nrules,1)*[0 nmfs*(1:k*ninps-1)];          % find index shift
rls = abs(rls) + shift;
g = mgrade(x(:)',mfs,domain);                           % calculate all grades
grades(:) = mask + sgn .* g(rls);                       % select using rls
if length(par) == 0,
  DOF(:) = feval(tnorm,[ones(1,k*nrules);grades']);     % non-parametric t-norm
else
  par
  DOF(:) = feval(tnorm,[ones(1,k*nrules);grades'],par); % parametric t-norm
end;
DOF = (ones(k,1)*w').*DOF';                             % weight by w

