% Demo - linguistic hedges.

% (c) Robert Babuska, March 1994

echo off
% Reset things before we get started.
clear
clc;clf;
format 
echo on
%       LINGUISTIC HEDGES

% Hedges are modifiers that change the meaning of linguistic terms
% described by fuzzy sets. They allow for modeling of composite terms
% such as "very high", "roughly zero", etc.

% Press any key to continue...
pause
clc
% First let us review basic operators that are used for modeling hedges:

%        normalization   An = Ax / Hgt A
%        concentration   Ac = Ax^2
%        dilution        Ad = 2.Ax - Ax^2

%        intensification Ai = 2.Ax^2                     for Ax < 0.5
%                        Ai = 1 - 2(1 - Ax)^2            otherwise

%        extensification Ae = (1/2.Ax)^0.5               for Ax < 0.5
%                        Ae = 1 - ((1 - Ax)/2)^0.5       otherwise

% Press any key to continue...
pause
clc
% The effect of these operators will be shown on the following fuzzy set:

X = -1:0.05:1; A = mgrade(X,[1 -0.8,-0.1,-0.1,0.5]);

Ac = conc(A);
Ad = dil(A);
Ai = intsf(A);
Ae = ext(A);

plot(X,A);

% Press any key to continue...
pause
echo off; clc
subplot(221); plot(X,[A;Ac]); title('conc A'); drawnow;
subplot(222); plot(X,[A;Ad]); title('dil A'); drawnow;
subplot(223); plot(X,[A;Ai]); title('int A'); drawnow;
subplot(224); plot(X,[A;Ae]); title('ext A'); drawnow;
echo on

% Press any key to continue...
pause
clc
% Here are several examples of frequently used hedges:

%        very A          = conc(A)
%        more or less A  = dil(A)
%        highly A        = A^3
%        roughly A       = dil(dil(A))
%        rather A        = intsf(conc(A))
%        slightly A      = intsf(normal(min(plush(A),not_za(very(A)))))

% Press any key to continue...
pause
echo off; clc
Ah  = highly(A);
Aro = roughly(A);
Ara = rather(A);
As  = slightly(A);

subplot(221); plot(X,[A;Ah]); title('highly A'); drawnow;
subplot(222); plot(X,[A;Aro]); title('roughly A'); drawnow;
subplot(223); plot(X,[A;Ara]); title('rather A'); drawnow;
subplot(224); plot(X,[A;As]); title('slightly A'); drawnow;
echo on

% Press any key to continue...
pause
clc
% Using hedges, we can construct such terms as "hot but not very hot",
% we can define new terms such as
% "middle aged" = "not very old and not very young", etc.
% Try it yourself.

% Press any key to quit...
pause
echo off; clc

