function  [sys, x0]  = sfunimpl(t,x,u,flag,figno,fs,domain,impl,par)
% SFUNIMPL an S-function which calculates fuzzy implication
%
%       This M-file is designed to be used in a SIMULINK S-function block.
%	
%       The input arguments are
%       figno:          Figure number (0 for no on-line plot).
%       fs:             Ouput fuzzy sets defined point-wise in rows.
%       domain:         Domain for output fuzzy sets.
%       impl:           Implication operator function (if [], default
%                       MIN is used).
%       par:            An additional parameter ([] for non-parametric
%                       operators).

% (c) Robert Babuska, December 1993

if abs(flag) == 2                       % state
   sys = x;

elseif flag == 3                        % output
  if length(impl) == 0,
      sys = min(u*ones(1,length(domain)),fs);
   elseif length(par) == 0,
      sys = feval(impl,u,fs);
  else
      sys = feval(impl,u,fs,par);
  end;
  sys = sys(:);
  if figno ~= 0,
    [nsets,npoints] = size(fs);
    ss = sys;
    if nsets == npoints,	% singletons
      npoints = 3*npoints;
      fss = fs; fs = zeros(nsets,npoints);
      fs(:,2:3:npoints) = fss; ss = fs;
      ss(:,2:3:npoints) = sys;
    end;
    for i = 1 : nsets,
      set(x(i),'ydata',fs(i,:));
    end;
    for i = 1 : nsets,
      set(x(nsets+i),'ydata',ss(i,:));
    end;
  end;

elseif flag  == 0,
  [nsets,npoints] = size(fs);
  % [cont, discr, outputs, inputs, roots, feedthrough]
  sys = [0, 2*nsets, prod(size(fs)), size(fs,1), 0, 1];
  if figno ~= 0,
        while ~any(get(0,'Children')==figno),
           figure('Visible','off');
        end
        set(figno,'Visible','off');
        oldfig = get(0,'CurrentFigure');
        set(0,'CurrentFigure',figno);
        clf;
        hold on;
        if nsets == npoints,	% singletons
          domain = ones(3,1)*domain; domain = domain(:)';
          fss = fs;
          fs = zeros(nsets,length(domain)); fs(:,2:3:length(domain)) = fss;
          x0 = plot(domain,fs,'EraseMode','normal');
          x0 = [x0; plot(domain,zeros(size(fs)),'EraseMode','background','LineWidth',5)];
        else
          x0 = plot(domain,fs,'EraseMode','normal');
          x0 = [x0; plot(domain,zeros(size(fs)),'EraseMode','background','LineWidth',3)];
        end;
        axis([min(domain) max(domain) 0 1]);
        hold off;
        set(figno,'BackingStore','off','Visible','on')
        set(0,'CurrentFigure',oldfig);
  else
    x0 = zeros(2*nsets,1);
  end;
else 
    sys = [];
end

