function r = impl_ma(a,b,s_norm,par)
%       R=IMPL_MA(A,B) Material implication A - > B, by default
%       defined as Zadeh's arithmetic rule of fuzzy implication
%       R = min(1, 1 - Ax + Bx).
%
%       R=IMPL_MA(A,B,S_NORM,PAR) General form of material implication
%       defined as R = (not A) + B, where + is a triangular co-norm and
%       PAR is a parameter for the co-norm.
%
%       See also IMPL_PR, IMPL_EX

% (c) Robert Babuska, March 1993

a = a(:); b = b(:).';                % make column and row vectors

if (nargin > 2)                         % s_norm supplied
  if (nargin < 4)
      eval(['r = ' s_norm '(not(a),b);']);
      return
  end;
  eval(['r = ' s_norm '(not(a),b,par);']);
  return;
end;

% Otherwise use Zadeh's arithmetic rule of fuzzy implication

am = length(a(:,1)); bn = length(b(1,:));
if ( am > 1 & bn > 1 )
    a = a*ones(1,bn); b = ones(am,1)*b;
end;

r = min(1,1-a+b);

