function c=s_du(a,b,alpha)
%       C=S_DU(A,B,ALPHA) The t-conorm proposed by Dubois and Prade.
%       ALPHA is from (0,1).
%
%       If A and B are both row or column vectors, then C
%       is also a row or column vector. If A is a column vector
%       and B a row vector, then a matrix C is returned,
%       representing the Cartesian product A x B.
%
%       S_DU(A,ALPHA) calculates the t-conorm between all rows in A.
%       In this case, ALPHA can be a vector of parameters.
%
%       See also SNORM, S_DO, S_LA, S_YA, S_FR, S_HA, S_SK.

% (c) Robert Babuska, March 1993

am = size(a,1); bn = size(b,2);
if nargin == 3,
  if ( am > 1 & bn > 1 )
    a=a*ones(1,bn); b=ones(am,1)*b;
  end;
  c = (a + b - a.*b - min(min(a,b),1-alpha))./max(max(1-a,1-b),alpha);
else
    for j = length(b) : -1 : 1,
      c(j,:) = a(1,:);
      for i = 2 : am,
        c(j,:) = (a(i,:) + c(j,:) - a(i,:).*c(j,:) - ...
            min(min(a(i,:),c(j,:)),1-b(j)))./max(max(1-a(i,:),1-c(j,:)),b(j));
      end;
    end;
end;
