function c=s_ha(a,b,gamma)
%       C=S_HA(A,B,GAMMA) The t-conorm proposed by Hamacher.
%       GAMMA > 0
%
%       If A and B are both row or column vectors, then C
%       is also a row or column vector. If A is a column vector
%       and B a row vector, then a matrix C is returned,
%       representing the Cartesian product A x B.
%
%       S_HA(A,GAMMA) calculates the t-conorm between all rows in A.
%       In this case, GAMMA can be a vector of parameters.
%
%       See also SNORM, S_DU, S_LA, S_YA, S_FR, S_DO, S_SK.

% (c) Robert Babuska, March 1993

am = size(a,1); bn = size(b,2);
if nargin == 3,
  if ( am > 1 & bn > 1 )
    a=a*ones(1,bn); b=ones(am,1)*b;
  end;
  c = (a + b - (2-gamma)*a.*b)./(1 - (1-gamma)*a.*b);
else
    for j = length(b) : -1 : 1,
      c(j,:) = a(1,:);
      for i = 2 : am,
        c(j,:) = (a(i,:) + c(j,:) - ...
		 (2-b(j))*a(i,:).*c(j,:))./(1 - (1-b(j))*a(i,:).*c(j,:));
      end;
    end;
end;
