function c=s_sk(a,b,p)
%       C=S_SK(A,B,P) The t-conorm proposed by Schweizer and Sklar.
%
%       If A and B are both row or column vectors, then C
%       is also a row or column vector. If A is a column vector
%       and B a row vector, then a matrix C is returned,
%       representing the Cartesian product A x B.
%
%       S_SK(A,P) calculates the t-conorm between all rows in A.
%       In this case, P can be a vector of parameters.
%
%       See also SNORM, S_DU, S_LA, S_YA, S_FR, S_HA, S_DO.

% (c) Robert Babuska, March 1993

am = size(a,1); bn = size(b,2);
if nargin == 3,
  if ( am > 1 & bn > 1 )
    a=a*ones(1,bn); b=ones(am,1)*b;
  end;
  c = (a.^p + b.^p - (a.^p).*(b.^p)).^(1/p);
else
    for j = length(b) : -1 : 1,
      c(j,:) = a(1,:);
      for i = 2 : am,
        c(j,:) = (a(i,:).^b(j) + c(j,:).^b(j) - ....
		 (a(i,:).^b(j)).*(c(j,:).^b(j))).^(1/b(j));
      end;
    end;
end;
