function c=s_ya(a,b,p)
%       C=S_YA(A,B,P) The t-conorm proposed by Yager.
%       P > 0
%
%       If A and B are both row or column vectors, then C
%       is also a row or column vector. If A is a column vector
%       and B a row vector, then a matrix C is returned,
%       representing the Cartesian product A x B.
%
%       S_YA(A,P) calculates the t-conorm between all rows in A.
%       In this case, P can be a vector of parameters.
%
%       See also SNORM, S_DU, S_LA, S_DO, S_FR, S_HA, S_SK.

% (c) Robert Babuska, March 1993


am = size(a,1); bn = size(b,2);
if nargin == 3,
  if ( am > 1 & bn > 1 )
    a=a*ones(1,bn); b=ones(am,1)*b;
  end;
  ab = a.^p + b.^p;
  if (ab < 1)
    c = ab.^(1/p);
  else
    c = min(1,ab.^(1/p));
  end;
else
    for j = length(b) : -1 : 1,
      c(j,:) = a(1,:);
      for i = 2 : am,
        ac = a(i,:).^b(j) + c(j,:).^b(j);
        if (ac < 1)
          c(j,:) = ac.^(1/b(j));
        else
          c(j,:) = min(1,ac.^(1/b(j)));
        end;
      end;
    end;
end;
