function c=t_du(a,b,alpha)
%       C=T_DU(A,B,ALPHA) The t-norm proposed by Dubois and Prade.
%       ALPHA is from (0,1).
%
%       If A and B are both row or column vectors, then C
%       is also a row or column vector. If A is a column vector
%       and B a row vector, then a matrix C is returned,
%       representing the Cartesian product A x B.
%
%       T_DU(A,ALPHA) calculates the t-norm between all rows in A.
%       In this case, ALPHA can be a vector of parameters.
%
%       See also TNORM, T_DO, T_LA, T_YA, T_FR, T_HA, T_SK.

% (c) Robert Babuska, March 1993

am = size(a,1); bn = size(b,2);
if nargin == 3,
  if ( am > 1 & bn > 1 )
    a=a*ones(1,bn); b=ones(am,1)*b;
  end;
  c = (a.*b)./max(max(a,b),alpha);
else
    for j = length(b) : -1 : 1,
      c(j,:) = a(1,:);
      for i = 2 : am,
        c(j,:) = (a(i,:).*c(j,:))./max(max(a(i,:),c(j,:)),b(j));
      end;
    end;
end;
