function c=t_la(a,b,lambda)
%       C=T_LA(A,B,LAMBDA) The lambda t-norm.
%       LAMBDA > -1
%
%       If A and B are both row or column vectors, then C
%       is also a row or column vector. If A is a column vector
%       and B a row vector, then a matrix C is returned,
%       representing the Cartesian product A x B.
%
%       T_LA(A,LAMBDA) calculates the t-norm between all rows in A.
%       In this case, LAMBDA can be a vector of parameters.
%
%       See also TNORM, T_DU, T_DO, T_YA, T_FR, T_HA, T_SK.

% (c) Robert Babuska, March 1993

am = size(a,1); bn = size(b,2);
if nargin == 3,
  if ( am > 1 & bn > 1 )
    a=a*ones(1,bn); b=ones(am,1)*b;
  end;
  c = max(0,(lambda + 1)*(a + b - 1) - lambda*a.*b);
else
    for j = length(b) : -1 : 1,
      c(j,:) = a(1,:);
      for i = 2 : am,
        c(j,:) = max(0,(b(j) + 1)*(a(i,:) + c(j,:) - 1) - b(j)*a(i,:).*c(j,:));
      end;
    end;
end;
