function [ret,x0,str,ts,xts]=fuzzylib(t,x,u,flag);
%FUZZYLIB	is the M-file description of the SIMULINK system named FUZZYLIB.
%	The block-diagram can be displayed by typing: FUZZYLIB.
%
%	SYS=FUZZYLIB(T,X,U,FLAG) returns depending on FLAG certain
%	system values given time point, T, current state vector, X,
%	and input vector, U.
%	FLAG is used to indicate the type of output to be returned in SYS.
%
%	Setting FLAG=1 causes FUZZYLIB to return state derivatives, FLAG=2
%	discrete states, FLAG=3 system outputs and FLAG=4 next sample
%	time. For more information and other options see SFUNC.
%
%	Calling FUZZYLIB with a FLAG of zero:
%	[SIZES]=FUZZYLIB([],[],[],0),  returns a vector, SIZES, which
%	contains the sizes of the state vector and other parameters.
%		SIZES(1) number of states
%		SIZES(2) number of discrete states
%		SIZES(3) number of outputs
%		SIZES(4) number of inputs
%		SIZES(5) number of roots (currently unsupported)
%		SIZES(6) direct feedthrough flag
%		SIZES(7) number of sample times
%
%	For the definition of other parameters in SIZES, see SFUNC.
%	See also, TRIM, LINMOD, LINSIM, EULER, RK23, RK45, ADAMS, GEAR.

% Note: This M-file is only used for saving graphical information;
%       after the model is loaded into memory an internal model
%       representation is used.

% the system will take on the name of this mfile:
sys = mfilename;
new_system(sys)
simver(1.3)
if (0 == (nargin + nargout))
     set_param(sys,'Location',[12,62,451,299])
     open_system(sys)
end;
set_param(sys,'algorithm',     'RK-45')
set_param(sys,'Start time',    '0.0')
set_param(sys,'Stop time',     '999999')
set_param(sys,'Min step size', '0.0001')
set_param(sys,'Max step size', '10')
set_param(sys,'Relative error','1e-3')
set_param(sys,'Return vars',   '')

add_block('built-in/S-Function',[sys,'/','Aggregation'])
set_param([sys,'/','Aggregation'],...
		'function name','sfunagr',...
		'parameters','figno, nsets, domain, agr, par',...
		'Mask Display','AGGR',...
		'Mask Type','Aggregation')
set_param([sys,'/','Aggregation'],...
		'Mask Dialogue','Aggregation|Figure (0 for no online plot): |Number of fuzzy sets:|Domain: |Aggregation operator: |Aggregation parameter:')
set_param([sys,'/','Aggregation'],...
		'Mask Translate','figno = @1; nsets = @2; domain = @3; agr = @4; par = @5;')
set_param([sys,'/','Aggregation'],...
		'Mask Help','Aggregates several fuzzy sets entering the block in one fuzzy set. Aggregation operator can be specified in the dialogue box.',...
		'Mask Entries','0\/4\/0:0.01:1\/''max''\/[]\/')
set_param([sys,'/','Aggregation'],...
		'position',[330,29,395,61])

add_block('built-in/S-Function',[sys,'/','Implication'])
set_param([sys,'/','Implication'],...
		'function name','sfunimpl',...
		'parameters','figno, fs, domain, impl, par',...
		'Mask Display','IMPL',...
		'Mask Type','Implication')
set_param([sys,'/','Implication'],...
		'Mask Dialogue','Implication|Figure (0 for no online plot):|Membership functions (defined point-wise on Domain): |Domain: |Implication operator: |Implication parameter:')
set_param([sys,'/','Implication'],...
		'Mask Translate','figno = @1; fs = @2; domain = @3; impl = @4; par = @5;',...
		'Mask Help','Given the fuzzy sets (block parameter) and degrees of fulfilment (block input), calculates the fuzzy implication.')
set_param([sys,'/','Implication'],...
		'Mask Entries','3\/mgrade(0:0.01:1,mfequ([0 1],1,4))\/0:0.01:1\/''impl_md''\/[]\/',...
		'position',[235,29,300,61])

add_block('built-in/S-Function',[sys,'/','DOF'])
set_param([sys,'/','DOF'],...
		'function name','sfundof',...
		'parameters','figno, rls, mfs, domain, tnorm, par',...
		'Mask Display','DOF',...
		'Mask Type','DOF')
set_param([sys,'/','DOF'],...
		'Mask Dialogue','Degree of fulfilment|Figure (0 for no plot): |Rule matrix: |Membership functions matrix: |Domain: |T-norm operator: |T-norm parameter:')
set_param([sys,'/','DOF'],...
		'Mask Translate','figno = @1; rls = @2; mfs = @3; domain = @4; tnorm = @5; par = @6;',...
		'Mask Help','Calculates degrees of fulfilment (truth values) of rule premises. For details see DOFTN.')
set_param([sys,'/','DOF'],...
		'Mask Entries','4\/[1 1; 1 2; 1 3; 1 4]\/mfequ([0 1],2,4)\/[]\/[]\/[]\/',...
		'position',[45,29,110,61])

add_block('built-in/S-Function',[sys,'/','Fuzzifier'])
set_param([sys,'/','Fuzzifier'],...
		'Font Name','Arial',...
		'function name','sfunfuzz',...
		'parameters','figno,mfs,rdom,idom',...
		'Mask Display','FUZZ',...
		'Mask Type','Fuzzification')
set_param([sys,'/','Fuzzifier'],...
		'Mask Dialogue','Fuzzification|Figure (0 for no online plot): |Membership functions matrix:|Reference domain ([] for continuous): |Input domain([] for crisp input):')
set_param([sys,'/','Fuzzifier'],...
		'Mask Translate','figno = @1; mfs = @2; rdom = @3; idom = @4;',...
		'Mask Help','Calculates membership degrees of an input into the specified fuzzy sets.',...
		'Mask Entries','1\/mfequ([0 1],1,4)\/[]\/[]\/')
set_param([sys,'/','Fuzzifier'],...
		'position',[140,29,205,61])

add_block('built-in/S-Function',[sys,'/','Fuzzy inference'])
set_param([sys,'/','Fuzzy inference'],...
		'function name','sfuninf',...
		'parameters','rls, mfs, def',...
		'Mask Display','INFER',...
		'Mask Type','Fuzzy inference block')
set_param([sys,'/','Fuzzy inference'],...
		'Mask Dialogue','MIN-MAX inference with \n fuzzy means defuzzification|Rule matrix: |Membership functions matrix: |Default output:',...
		'Mask Translate','rls = @1; mfs = @2; def = @3;')
set_param([sys,'/','Fuzzy inference'],...
		'Mask Help','Fuzzy inference block can be used as a direct fuzzy controller or as a fuzzy supervisor. For details on dialog parameters see INFER.')
set_param([sys,'/','Fuzzy inference'],...
		'Mask Entries','[1 1 5; 1 2 6; 1 3 7; 1 4 8]\/[mfequ([0 1],3,4); mfequ([0 1],1,4)]\/0\/',...
		'position',[45,89,110,121])

add_block('built-in/S-Function',[sys,'/','Composition'])
set_param([sys,'/','Composition'],...
		'Font Name','Arial',...
		'function name','sfunrel',...
		'parameters','R',...
		'Mask Display','MAX-MIN',...
		'Mask Type','Composition',...
		'Mask Dialogue','Relational composition|Fuzzy relation:')
set_param([sys,'/','Composition'],...
		'Mask Translate','R = @1;',...
		'Mask Help','Calculates MAX-MIN relational composition.',...
		'Mask Entries','R\/',...
		'position',[155,87,230,123])


%     Subsystem  'Bar Scope'.

new_system([sys,'/','Bar Scope'])
set_param([sys,'/','Bar Scope'],'Location',[4,68,278,261])

add_block('built-in/S-Function',[sys,'/',['Bar Scope/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['Bar Scope/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfunbar',...
		'parameters','figno,nbars,ymax,color',...
		'position',[130,55,180,75])

add_block('built-in/Inport',[sys,'/','Bar Scope/x'])
set_param([sys,'/','Bar Scope/x'],...
		'position',[65,55,85,75])
add_line([sys,'/','Bar Scope'],[90,65;125,65])
set_param([sys,'/','Bar Scope'],...
		'Mask Display','BAR',...
		'Mask Type','Bar graph scope.')
set_param([sys,'/','Bar Scope'],...
		'Mask Dialogue','Bar graph scope using MATLAB graph window.\nEnter number of bars and line type.|Figure:|Number of bars:|y-max:|Line type (rgbw-:*):')
set_param([sys,'/','Bar Scope'],...
		'Mask Translate','figno = @1; nbars = @2; ymax = @3; color = @4; ',...
		'Mask Help','This block plots to the MATLAB graph window a bar graph. Look at the m-file sfunbar.m to see how it works.')
set_param([sys,'/','Bar Scope'],...
		'Mask Entries','1\/5\/1\/''y-''\/')


%     Finished composite block 'Bar Scope'.

set_param([sys,'/','Bar Scope'],...
		'Drop Shadow',4,...
		'position',[55,159,100,191])


%     Subsystem  'FScope'.

new_system([sys,'/','FScope'])
set_param([sys,'/','FScope'],'Location',[4,68,278,261])

add_block('built-in/Inport',[sys,'/','FScope/x'])
set_param([sys,'/','FScope/x'],...
		'position',[65,55,85,75])

add_block('built-in/S-Function',[sys,'/',['FScope/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['FScope/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfunfset',...
		'parameters','figno,nsets,domain,color',...
		'position',[130,55,180,75])
add_line([sys,'/','FScope'],[90,65;125,65])
set_param([sys,'/','FScope'],...
		'Mask Display','plot(0,0,100,100,[90,10,10,10,90,90,10],[65,65,90,40,40,90,90],[90,78,69,54,40,31,25,10],[77,60,48,46,56,75,81,84])',...
		'Mask Type','Fuzzy scope.')
set_param([sys,'/','FScope'],...
		'Mask Dialogue','Bar graph scope using MATLAB graph window.\nEnter number of bars and line type.|Figure:|Number fuzzy sets:|domain:|Line type (rgbw-:*):')
set_param([sys,'/','FScope'],...
		'Mask Translate','figno = @1; nsets = @2; domain = @3; color = @4; ',...
		'Mask Help','This block plots fuzzy set(s) to the MATLAB graph window. Look at the m-file sfunfset.m to see how it works.')
set_param([sys,'/','FScope'],...
		'Mask Entries','3\/1\/-10:0.1:10\/''g''\/')


%     Finished composite block 'FScope'.

set_param([sys,'/','FScope'],...
		'Drop Shadow',4,...
		'position',[150,156,180,194])


%     Subsystem  'GScope'.

new_system([sys,'/','GScope'])
set_param([sys,'/','GScope'],'Location',[4,68,278,261])

add_block('built-in/Inport',[sys,'/','GScope/x'])
set_param([sys,'/','GScope/x'],...
		'position',[65,55,85,75])

add_block('built-in/S-Function',[sys,'/',['GScope/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['GScope/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfunplot',...
		'parameters','figno,ax, color',...
		'position',[130,55,180,75])
add_line([sys,'/','GScope'],[90,65;125,65])
set_param([sys,'/','GScope'],...
		'Mask Display','plot(0,0,100,100,[90,10,10,10,90,90,10],[65,65,90,40,40,90,90],[90,78,69,54,40,31,25,10],[77,60,48,46,56,75,81,84])',...
		'Mask Type','Graph scope.')
set_param([sys,'/','GScope'],...
		'Mask Dialogue','Graph scope using MATLAB graph window.\nEnter figure, plotting ranges and line type.|Figure:|Time range:|y-min:|y-max:|Line type (rgbw-:*):')
set_param([sys,'/','GScope'],...
		'Mask Translate','figno = @1; color = @5; ax = [0, @2, @3, @4];')
set_param([sys,'/','GScope'],...
		'Mask Help','This block plots to the MATLAB graph window and can be used as an improved version of the Graph Scope block. Look at the m-file sfunplot.m to see how it works.')
set_param([sys,'/','GScope'],...
		'Mask Entries','1\/1\/0\/1\/''y-''\/')


%     Finished composite block 'GScope'.

set_param([sys,'/','GScope'],...
		'Drop Shadow',4,...
		'position',[235,156,265,194])

add_block('built-in/S-Function',[sys,'/','Defuzzifier'])
set_param([sys,'/','Defuzzifier'],...
		'function name','sfundef',...
		'parameters','figno,domain, def, par1, par2',...
		'Mask Display','DEFUZ',...
		'Mask Type','Defuzzification')
set_param([sys,'/','Defuzzifier'],...
		'Mask Dialogue','Defuzzification|Figure (0 for no online plot): |Domain: |Defuzzification operator: |Parameter1:|Parameter2:',...
		'Mask Translate','figno = @1; domain = @2; def = @3; par1 = @4; par2 = @5;')
set_param([sys,'/','Defuzzifier'],...
		'Mask Help','Defuzzifies a fuzzy set entering the block, producing a crisp output.',...
		'Mask Entries','2\/0:0.01:1\/''coa''\/[]\/[]\/',...
		'position',[260,89,325,121])

add_block('built-in/S-Function',[sys,'/','Sugeno'])
set_param([sys,'/','Sugeno'],...
		'function name','sfunsug',...
		'parameters','rls, mfs, p, def',...
		'Mask Display','SUGENO',...
		'Mask Type','Sugeno''s fuzzy inference')
set_param([sys,'/','Sugeno'],...
		'Mask Dialogue','Sugeno-Takagi inference|Premise matrix: |Membership functions matrix: |Consequent parameters: |Default output:',...
		'Mask Translate','rls = @1; mfs = @2; p = @3; def = @4;')
set_param([sys,'/','Sugeno'],...
		'Mask Help','Performs Sugeno-Takagi''s inference. For details see SUGVAL.',...
		'Mask Entries','[ones(7,1) (1:7)'']\/mfequ(-1:0.1:1,1,7)\/p\/0\/',...
		'position',[345,89,420,121])


%     Subsystem  'GScope2'.

new_system([sys,'/','GScope2'])
set_param([sys,'/','GScope2'],'Location',[4,85,278,278])

add_block('built-in/Constant',[sys,'/','GScope2/Constant'])
set_param([sys,'/','GScope2/Constant'],...
		'Value','0',...
		'position',[50,55,70,75])

add_block('built-in/S-Function',[sys,'/',['GScope2/S-function',13,'M-file which ',13,'pauses']])
set_param([sys,'/',['GScope2/S-function',13,'M-file which ',13,'pauses']],...
		'function name','sfunpaus',...
		'parameters','sec',...
		'position',[130,55,180,75])
add_line([sys,'/','GScope2'],[75,65;125,65])
set_param([sys,'/','GScope2'],...
		'Mask Display','PAUSE',...
		'Mask Type','Pause',...
		'Mask Dialogue','Pause|Pause for second(s):',...
		'Mask Translate','sec  = @1;',...
		'Mask Entries','0\/')


%     Finished composite block 'GScope2'.

set_param([sys,'/','GScope2'],...
		'hide name',0,...
		'Drop Shadow',4,...
		'position',[315,164,390,196])

drawnow

% Return any arguments.
if (nargin | nargout)
	% Must use feval here to access system in memory
	if (nargin > 3)
		if (flag == 0)
			eval(['[ret,x0,str,ts,xts]=',sys,'(t,x,u,flag);'])
		else
			eval(['ret =', sys,'(t,x,u,flag);'])
		end
	else
		[ret,x0,str,ts,xts] = feval(sys);
	end
else
	drawnow % Flash up the model and execute load callback
end
